@@ -1,6 +1,6 @@
 /* @flow */
 import type {Url, Src, Locator} from './common';
-import {format_dt, Methods, unwrap} from './common';
+import {format_dt, Methods, unwrap, safeSetInnerHTML} from './common';
 
 // TODO need to pass document??
 
@@ -24,6 +24,9 @@ type Params = {
     relative: boolean;
 }
 
+
+const HTML_MARKER = '!html ';
+
 export class Binder {
     doc: Document;
 
@@ -48,6 +51,17 @@ export class Binder {
         return res;
     }
 
+    error(
+        parent: HTMLElement,
+        message: string,
+    ) {
+        const child = this.makeChild.bind(this);
+        const tchild = this.makeTchild.bind(this); // TODO still necessary??
+
+        const item = child(parent, 'div', ['error']);
+        tchild(item, "ERROR: " + message);
+    }
+
     render(
         parent: HTMLElement,
         dates: string,
@@ -77,7 +91,12 @@ export class Binder {
         const date_c = child(dt_c, 'span', ['date']);
         item.setAttribute('sources', tags.join(' '));
 
-        tchild(relative_c, '⤑⤑');
+        const child_link = child(relative_c, 'a');
+        // ugh. not sure why opening in new tab doesn't work :(
+        // https://stackoverflow.com/questions/12454382/target-blank-is-not-working-in-firefox/12454474#12454474
+        // child_link.target = '_blank';
+        child_link.href = original_url;
+        tchild(child_link, '➤➤');
 
         const idx_c = child(tags_c, 'span', ['index']);
         idx_c.title = 'index (for easier match against highlights)';
@@ -94,11 +113,10 @@ export class Binder {
         tchild(time_c, times);
         dt_c.setAttribute('title', 'search around');
         dt_c.onclick = () => {
-            const ts = timestamp.getTime() / 1000;
+            const utc_ts = timestamp.valueOf();
             chrome.runtime.sendMessage({
-                // TODO hopefully the timestamp is properly captured??
                 method   : Methods.SEARCH_VISITS_AROUND,
-                timestamp: ts,
+                utc_timestamp: utc_ts,
             });
 
             return true;
@@ -106,10 +124,17 @@ export class Binder {
 
         /* TODO locator could jump into the file? */
         if (context != null) {
-            const ctx_c = child(item, 'div', ['context']);
-            for (const line of context.split('\n')) {
-                tchild(ctx_c, line);
-                child(ctx_c, 'br');
+            const ctx_c = child(item, 'div', ['context'])
+
+            let ctx = context;
+            if (ctx.startsWith(HTML_MARKER)) {
+                ctx = context.substring(HTML_MARKER.length)
+                safeSetInnerHTML(ctx_c, ctx);
+            } else { // plaintext
+                for (const line of ctx.split('\n')) {
+                    tchild(ctx_c, line)
+                    child(ctx_c, 'br')
+                }
             }
         }
 
@@ -144,6 +169,8 @@ export class Binder {
             det.appendChild(doc.createTextNode(context));
             */
         }
+
+        // right, this is for search..
         if (normalised_url != null) {
             const nurl_c = child(item, 'div', ['normalised_url']);
             const link = child(nurl_c, 'a');
