@@ -10,13 +10,9 @@ import logging
 from functools import lru_cache
 import traceback
 import pytz
+import warnings
 
-from .normalise import normalise_url
-from .cannon import CanonifyException
-
-
-# TODO not sure if should keep it? it's throwing some warnings
-import dateparser # type: ignore
+from .cannon import CanonifyException, canonify
 
 
 T = TypeVar('T')
@@ -25,8 +21,8 @@ Res = Union[T, Exception]
 PathIsh = Union[str, Path]
 
 Url = str
-Source = str
-DatetimeIsh = Union[datetime, date, str]
+SourceName = str
+DatetimeIsh = Union[datetime, date]
 Context = str
 Second = int
 
@@ -36,7 +32,7 @@ class Loc(NamedTuple):
     href: Optional[str]=None
 
     @classmethod
-    def make(cls, title: str, href: Optional[str]=None):
+    def make(cls, title: str, href: Optional[str]=None) -> 'Loc':
         return cls(title=title, href=href)
 
     @classmethod
@@ -75,26 +71,25 @@ class DbVisit(NamedTuple):
     orig_url: Url
     dt: datetime
     locator: Loc
-    src: Optional[Source] = None
+    src: Optional[SourceName] = None
     context: Optional[Context] = None
     duration: Optional[Second] = None
 
     @staticmethod
-    def make(p: PreVisit, src: Source) -> Res['DbVisit']:
+    def make(p: PreVisit, src: SourceName) -> Res['DbVisit']:
         try:
             if isinstance(p.dt, datetime):
                 dt = p.dt
             elif isinstance(p.dt, date):
+                # TODO that won't be with timezone..
                 dt = datetime.combine(p.dt, datetime.min.time()) # meh..
-            elif isinstance(p.dt, str):
-                dt = dateparser.parse(p.dt)
             else:
                 raise AssertionError(f'unexpected date: {p.dt}, {type(p.dt)}')
         except Exception as e:
             return e
 
         try:
-            nurl = normalise_url(p.url)
+            nurl = canonify(p.url)
         except Exception as e:
             return e
 
@@ -122,7 +117,7 @@ def make_filter(thing) -> Filter:
         return thing
 
 
-def get_logger():
+def get_logger() -> logging.Logger:
     return logging.getLogger("promnesia")
 
 # TODO do i really need to inherit this??
@@ -156,7 +151,7 @@ class History(Sized):
     def add_filter(cls, filterish):
         cls.FILTERS.append(make_filter(filterish))
 
-    def __init__(self, *, src: Source):
+    def __init__(self, *, src: SourceName):
         self.vmap: Dict[PreVisit, DbVisit] = {}
         # TODO err... why does it map from previsit???
         self.logger = get_logger()
@@ -226,11 +221,20 @@ def get_tmpdir():
     tdir = tempfile.TemporaryDirectory(suffix="promnesia")
     return tdir
 
-@lru_cache(1)
-def _get_extractor():
+# TODO use mypy literal?
+Syntax = str
+
+
+@lru_cache(None)
+def _get_extractor(syntax: Syntax):
     from urlextract import URLExtract # type: ignore
     u = URLExtract()
     # https://github.com/lipoja/URLExtract/issues/13
+    if syntax in {'org', 'orgmode', 'org-mode'}: # TODO remove hardcoding..
+        u._stop_chars_right |= {'[', ']'}
+        u._stop_chars_left  |= {'[', ']'}
+    elif syntax in {'md', 'markdown'}:
+        pass
     # u._stop_chars_right |= {','}
     # u._stop_chars_left  |= {','}
     return u
@@ -245,9 +249,8 @@ def sanitize(url: str) -> str:
         url = url.strip(')')
     return url
 
-
 # TODO sort just in case? not sure..
-def _extract_urls(s: str) -> List[str]:
+def _extract_line_urls(*, s: str, syntax: Syntax) -> List[str]:
     # TODO unit test for escaped urls.. or should it be in normalise instead?
     if len(s.strip()) == 0:
         return [] # optimize just in case..
@@ -257,7 +260,7 @@ def _extract_urls(s: str) -> List[str]:
     # TODO fuck. doesn't look like it's handling multiple urls in same line well...
     # ("python.org/one.html python.org/two.html",
     # hopefully ok... I guess if there are spaces in url we are fucked anyway..
-    extractor = _get_extractor()
+    extractor = _get_extractor(syntax=syntax)
     urls: List[str] = []
     for x in s.split():
         urls.extend(extractor.find_urls(x))
@@ -265,14 +268,18 @@ def _extract_urls(s: str) -> List[str]:
     return [sanitize(u) for u in urls]
 
 
-def extract_urls(s: str) -> List[str]:
-    return list(itertools.chain.from_iterable(map(_extract_urls, s.splitlines())))
+def _extract_urls(*, s: str, syntax: Syntax) -> Iterable[Url]:
+    for line in s.splitlines():
+        yield from _extract_line_urls(s=line, syntax=syntax)
 
 
+def extract_urls(s: str, syntax: Syntax='') -> List[Url]:
+    return list(_extract_urls(s=s, syntax=syntax))
+
+
+# TODO maybe belongs to HPI?
 def from_epoch(ts: int) -> datetime:
-    res = datetime.utcfromtimestamp(ts)
-    res.replace(tzinfo=pytz.utc)
-    return res
+    return datetime.fromtimestamp(ts, tz=pytz.utc)
 
 
 class PathWithMtime(NamedTuple):
@@ -287,16 +294,23 @@ class PathWithMtime(NamedTuple):
         )
 
 
-class Indexer:
-    def __init__(self, ff, *args, src: str, **kwargs) -> None:
+class Source:
+    # TODO make sure it works with empty src?
+    # TODO later, make it properly optional?
+    def __init__(self, ff, *args, src: SourceName='', name: SourceName='', **kwargs) -> None:
         self.ff = ff
         self.args = args
         self.kwargs = kwargs
-        self.src = src
+        if src is not None:
+            warnings.warn("'src' argument is deprecated, please use 'name' instead", DeprecationWarning)
+        self.src = name or src
+
+# TODO deprecated
+Indexer = Source
 
 
 # TODO do we really need it?
-def previsits_to_history(extractor, *, src: Source) -> Tuple[List[DbVisit], List[Exception]]:
+def previsits_to_history(extractor, *, src: SourceName) -> Tuple[List[DbVisit], List[Exception]]:
     ex = extractor
     # TODO isinstance wrapper?
     # TODO make more defensive?
@@ -320,9 +334,10 @@ def previsits_to_history(extractor, *, src: Source) -> Tuple[List[DbVisit], List
     for p in previsits:
         if isinstance(p, Exception):
             errors.append(p)
-            parts = ['indexer emitted exception']
-            parts.extend(traceback.format_tb(p.__traceback__))
-            logger.error('\n'.join(parts))
+            parts = ['indexer emitted exception\n']
+            # eh, exception type is ignored by format_exception completely, apparently??
+            parts.extend(traceback.format_exception(Exception, p, p.__traceback__))
+            logger.error(''.join(parts))
             continue
 
         # TODO check whether it's filtered before construction? probably doesn't really impact
@@ -336,6 +351,7 @@ def previsits_to_history(extractor, *, src: Source) -> Tuple[List[DbVisit], List
     return h.visits, errors
 
 
+# not sure if necessary anymore?
 # NOTE: used in configs...
 def last(path: PathIsh, *parts: str) -> Path:
     pp = os.path.join(str(path), *parts)
