@@ -1,6 +1,6 @@
 import importlib
 import sys
-from contextlib import contextmanager, nullcontext
+from contextlib import contextmanager
 
 
 @contextmanager
