@@ -91,7 +91,7 @@ def get_hotkey(driver, cmd: str) -> str:
         # TODO remove hardcoding somehow...
         # perhaps should be extracted somewhere..
         cmd_map = {
-            'show_dots'              : 'Ctrl+Alt+v',
+            Command.SHOW_DOTS        : 'Ctrl+Alt+v',
             '_execute_browser_action': 'Ctrl+Alt+e',
             'search'                 : 'Ctrl+Alt+h',
         }
@@ -221,7 +221,10 @@ def get_window_id(driver):
         # ugh nothing in capabilities...
         pid = check_output(['pgrep', '-f', 'chrome.*enable-automation']).decode('utf8').strip()
     # https://askubuntu.com/a/385037/427470
+    return get_wid_by_pid(pid)
 
+
+def get_wid_by_pid(pid: str):
     wids = check_output(['xdotool', 'search', '--pid', pid]).decode('utf8').splitlines()
     wids = [w.strip() for w in wids if len(w.strip()) > 0]
 
@@ -282,9 +285,16 @@ class TestHelper(NamedTuple):
     def activate(self):
         self.command(Command.ACTIVATE)
 
-    def show_visited(self):
+    def mark_visited(self):
         self.command(Command.SHOW_DOTS)
 
+    def wid(self) -> str:
+        return get_window_id(self.driver)
+
+    def screenshot(self, path):
+        # ugh, webdriver's save_screenshot doesn't behave well with frames
+        check_call(['import', '-window', self.wid(), path])
+
 
 def confirm(what: str):
     import click # type: ignore
@@ -310,7 +320,7 @@ def _test_helper(tmp_path, indexer, test_url: Optional[str], show_dots: bool=Fal
         yield TestHelper(driver=driver)
 
 class Command:
-    SHOW_DOTS = 'show_dots'
+    SHOW_DOTS = 'mark_visited'
     ACTIVATE  = '_execute_browser_action'
     SEARCH    = 'search'
 # TODO assert this against manifest?
