@@ -104,8 +104,7 @@ def post(*args):
     return json.loads(check_output(cmd).decode('utf8'))
 
 
-def test_query_instapaper(tmp_path):
-    tdir = Path(tmp_path)
+def test_query_instapaper(tdir):
     index_hypothesis(tdir)
     test_url = "http://www.e-flux.com/journal/53/59883/the-black-stack/"
     with wserver(db=tdir / 'promnesia.sqlite') as helper:
@@ -123,8 +122,7 @@ def test_visits(tmp_path):
             assert len(response['visits']) == 1
 
 
-def test_search(tmp_path):
-    tdir = Path(tmp_path)
+def test_search(tdir):
     index_hypothesis(tdir)
     test_url = "http://www.e-flux.com"
     with wserver(db=tdir / 'promnesia.sqlite') as helper:
@@ -142,7 +140,8 @@ def test_visited(tmp_path):
 def test_search_around(tmp_path):
     tdir = Path(tmp_path)
     index_hypothesis(tdir)
-    test_ts = int(datetime.strptime("2017-05-22T10:58:14.082375+00:00", '%Y-%m-%dT%H:%M:%S.%f%z').timestamp())
+    dt = pytz.utc.localize(datetime.strptime("2017-05-22T10:58:14.082375", '%Y-%m-%dT%H:%M:%S.%f'))
+    test_ts = int(dt.timestamp())
     # test_ts = int(datetime(2016, 12, 13, 12, 31, 4, 229275, tzinfo=pytz.utc).timestamp())
     # TODO hmm. perhaps it makes more sense to run query in different process and server in main process for testing??
     with wserver(db=tdir / 'promnesia.sqlite') as helper:
@@ -166,3 +165,11 @@ def test_visits_hier(tdir):
         response = post(f'http://localhost:{helper.port}/visits', f'url={test_url}')
         assert {v['context'] for v in response['visits']} == {'parent url', 'Some context'}
 
+
+def test_status(tdir):
+    with wserver(db='/does/not/exist') as helper:
+        response = post(f'http://localhost:{helper.port}/status')
+        assert response['db'] == None # defensive, it doesn't exist
+        version = response['version']
+        assert version is not None
+        assert len(version.split('.')) >= 2 # random check..
